// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "ServerSocket.h"

#include "Socket.h"

#include <unistd.h>
#include <sys/socket.h>

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::ServerSocket;
using hu::varadiistvan::scpl::io::Acceptor;

//------------------------------------------------------------------------------

ServerSocket::ServerSocket(int domain, int type, Waiter* waiter, int protocol) :
    Waitable(waiter, Socket::socket(domain, type, protocol))
{
    if (fd<0) setErrorCodeFromErrno();
    else if (waiter!=0) setNonBlocking(fd);
}

//------------------------------------------------------------------------------

ServerSocket::~ServerSocket()
{
    if (fd>=0) ::close(fd);
}

//------------------------------------------------------------------------------

bool ServerSocket::bind(const struct sockaddr* addr, size_t addrlen,
                        bool reuseaddr)
{
    if (reuseaddr) {
        int reuse = 1;
        if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse))<0) {
            setErrorCodeFromErrno();
            return false;
        }
    }

    if (::bind(fd, addr, addrlen)<0) {
        setErrorCodeFromErrno();
        return false;
    }

    return true;
}

//------------------------------------------------------------------------------

bool ServerSocket::listen(int backlog)
{
    if (::listen(fd, backlog)<0) {
        setErrorCodeFromErrno();
        return false;
    } else {
        return true;
    }
}

//------------------------------------------------------------------------------

int ServerSocket::accept(struct sockaddr* addr, size_t* addrlen)
{
    socklen_t alen = (addrlen==0) ? 0 : *addrlen;
    int result = ::accept(fd, addr, (addrlen==0) ? 0 : &alen);
    if (result<0 && errno!=EAGAIN && errno!=EWOULDBLOCK) {
        setErrorCodeFromErrno();
    } else {
        if (addrlen!=0) *addrlen = alen;
    }
    return result;
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
